local g = golly()

gridtype = "P"
xsize = 512
ysize = 512
dens = 30
runtime = 500
local RefreshCounter = 0

local TorusString = ":"..gridtype..xsize..","..ysize
local SelRect = {math.ceil(xsize/-2),math.ceil(ysize/-2),xsize,ysize}

g.addlayer() --make second layer for processing

while g.numlayers() > 1 do
g.setlayer(g.numlayers()-1)
g.select(g.getrect())
if not g.empty() then
	g.clear(0)
end
g.dellayer()
end

g.setlayer(0)
g.select(g.getrect())
if not g.empty() then
	g.clear(0)
end
g.addlayer()
g.setlayer(0)
g.new("Searching")

local function IsolateOneOscillator()
	--change rule to isolate
	g.setrule("isolate"..TorusString)
	--change a random state 1 cell to state 4
	local cells = g.getcells(g.getrect())
	--find a state 1 cell
	local found = false
	local todo
	for i = 1,#cells,3 do
		if cells[i+2] == 1 then
			found = true
			todo = i
			break;
		end
	end
	if found then
		g.setcell(cells[todo],cells[todo+1],4)
	else
		--g.show("None found")
		g.select(g.getrect())
		if not g.empty() then
			g.clear(0)
		end
	end
	--run for 200 gens
	g.run(200)

	--copy entire pattern into layer 2
	--g.select({math.ceil(xsize/-2),math.ceil(ysize/-2),xsize,ysize})
	local cells = g.getcells(g.getrect())
	g.setlayer(1)
	g.setrule("isolate"..TorusString)
	g.putcells(cells)

	--remove all state 3,4,5 cells from layer 1
	g.setlayer(0)
	g.setrule("remove345"..TorusString)
	g.step()
	g.setrule("greycells"..TorusString)
	--remove all state 1,2 cells from layer 2
	g.setlayer(1)
	g.setrule("remove12"..TorusString)
	g.step()
	g.setrule("convert"..TorusString)
	g.step()
	g.setrule("greycells"..TorusString)
	--an oscillator has now been moved from layer 1 to layer 2
end

local function FindItsPeriod()
	g.setlayer(1)
	--oscar
	local pppppppppp = 0

-- initialize lists
local ohashlist = {}     -- for pattern hash values
local ogenlist = {}      -- corresponding generation counts
local opoplist = {}      -- corresponding population counts
local oboxlist = {}      -- corresponding bounding boxes

----------------------------------------------------------------------

local function ooscillating()
    -- return true if the pattern is empty, stable or oscillating
    
    -- first get current pattern's bounding box
    local opbox = g.getrect()
    if #opbox == 0 then
        --g.show("The pattern is empty.")
        return true
    end
    
    local oh = g.hash(opbox)
    
    -- determine where to insert h into hashlist
    local opos = 1
    local olistlen = #ohashlist
    while opos <= olistlen do
        if oh > ohashlist[opos] then
            opos = opos + 1
        elseif oh < ohashlist[opos] then
            -- shorten lists and append info below
            for oi = 1, olistlen - opos + 1 do
                table.remove(ohashlist)
                table.remove(ogenlist)
                table.remove(opoplist)
                table.remove(oboxlist)
            end
            break
        else
            -- h == hashlist[pos] so pattern is probably oscillating, but just in
            -- case this is a hash collision we also compare pop count and box size
            local orect = oboxlist[opos]
            if tonumber(g.getpop()) == opoplist[opos] and opbox[3] == orect[3] and opbox[4] == orect[4] then
                local operiod = tonumber(g.getgen()) - ogenlist[opos]
                
                if ohasB0notS8 and (operiod % 2) > 0 and
                   opbox[1] == orect[1] and opbox[2] == orect[2] and
                   opbox[3] == orect[3] and opbox[4] == orect[4] then
                    -- ignore this hash value because B0-and-not-S8 rules are
                    -- emulated by using different rules for odd and even gens,
                    -- so it's possible to have identical patterns at gen G and
                    -- gen G+p if p is odd
                    return false
                end
                
                if operiod == 1 then
                    if opbox[1] == orect[1] and opbox[2] == orect[2] and
                       opbox[3] == orect[3] and opbox[4] == orect[4] then
                        --g.show("The pattern is stable.")
                    else
                        --show_spaceship_speed(1, 0, 0)
                    end
                elseif opbox[1] == orect[1] and opbox[2] == orect[2] and
                       opbox[3] == orect[3] and opbox[4] == orect[4] then
               		 pppppppppp = operiod
                else
                    local odeltax = math.abs(orect[1] - opbox[1])
                    local odeltay = math.abs(orect[2] - opbox[2])
                    --show_spaceship_speed(period, deltax, deltay)
                end
                return true
            else
                -- look at next matching hash value or insert if no more
                opos = opos + 1
            end
        end
    end
    
    -- store hash/gen/pop/box info at same position in various lists
    table.insert(ohashlist, opos, oh)
    table.insert(ogenlist, opos, tonumber(g.getgen()))
    table.insert(opoplist, opos, tonumber(g.getpop()))
    table.insert(oboxlist, opos, opbox)
    
    return false
end

local function ofit_if_not_visible()
    -- fit pattern in viewport if not empty and not completely visible
    local oor = g.getrect()
    if #oor > 0 and not g.visrect(oor) then g.fit() end
end

--g.show("Finding period... (hit escape to abort)")

local ooldsecs = os.clock()
while not ooscillating() do
    g.run(1)
    local onewsecs = os.clock()
    if onewsecs - ooldsecs >= 1.0 then   -- show pattern every second
        ooldsecs = noewsecs
	ofit_if_not_visible()
        g.update()
    end
end
ofit_if_not_visible()
	return pppppppppp
end

local function SaveIt(Period)
	if Period > 17 and Period ~= 25 and Period ~= 28 then
		--it's interesting, so save it
		--standardise name
		local minpop = tonumber(g.getpop())
		for i = 1,Period do
			g.step()
			if minpop > tonumber(g.getpop()) then
				minpop = tonumber(g.getpop())
			end
		end
		local hashes = {}
		for i = 1,Period do
			g.step()
			for x = 1,4 do
				g.select(SelRect)
				g.rotate(0)
				hashes[#hashes+1] = g.hash(g.getrect())
			end
			g.flip(1)
			for x = 1,4 do
				g.select(SelRect)
				g.rotate(0)
				hashes[#hashes+1] = g.hash(g.getrect())
			end
			g.flip(0)
			for x = 1,4 do
				g.select(SelRect)
				g.rotate(0)
				hashes[#hashes+1] = g.hash(g.getrect())
			end
			g.flip(1)
			for x = 1,4 do
				g.select(SelRect)
				g.rotate(0)
				hashes[#hashes+1] = g.hash(g.getrect())
			end
		end
		table.sort(hashes)
		--g.note(table.concat(hashes,"\n"))
		local Name = tostring(hashes[1]).." - "..tostring(minpop)--.." - "..tostring(os.clock())
		--save
		g.show("Found P"..Period..": "..Name)
		g.save("/AllOscs/"..Period.." - "..Name..".rle","rle")
		if Period > 100 then
			g.save("/BIG/"..Period.." - "..Name..".rle","rle")
		else
			g.save("/"..tostring(Period).."/"..Name..".rle","rle")
		end
	else
		--it's not interesting, don't waste space on it
	end
	g.select(g.getrect())
	if not g.empty() then
		g.clear(0)
	end
	g.setlayer(0)
	g.setrule("greycells"..TorusString)
end

local function process()
	g.setbase(2)
	g.setstep(0)
	local ThereAreOscillators = not g.empty()
	g.setrule("convert"..TorusString)
	g.step()
	g.setrule("greycells"..TorusString)
	while ThereAreOscillators do
		IsolateOneOscillator()
		local Period = FindItsPeriod()
		SaveIt(Period)
		ThereAreOscillators = not g.empty()
	end
end

while true do

g.setgen(0)
g.setbase(2)

g.setpos(0,math.ceil(ysize/-2))
g.setrule("greycells"..TorusString)
g.select({math.ceil(xsize/-2),math.ceil(ysize/-2),xsize,ysize})
g.randfill(dens)
g.setstep(9)
g.step()

g.setrule("findosc"..TorusString)
g.run(100)

g.setrule("convert"..TorusString)
g.setstep(0)
g.step()

g.setrule("greycells"..TorusString)
g.step()

g.setrule("findosc"..TorusString)
g.run(100)



          --  g.show("Oscillator detected (period = "..period.." = "..factorise(period)..")")
			--local HF = FHPF(period)
			--if HF > 17 then
				process()
		--	end

RefreshCounter = RefreshCounter + 1
if RefreshCounter%50 == 0 then
	while g.numlayers() > 1 do
	g.setlayer(g.numlayers()-1)
	g.select(g.getrect())
	if not g.empty() then
		g.clear(0)
	end
	g.dellayer()
	end
	
	g.setlayer(0)
	g.select(g.getrect())
	if not g.empty() then
		g.clear(0)
	end
	g.addlayer()
	g.setlayer(0)
	g.new("Searching")
end
end